/*
sleeplight.c
Apple-style LED sleep light simulator.

Written by Windell Oskay, http://www.evilmadscientist.com/

 Copyright 2009 Windell H. Oskay
 Distributed under the terms of the GNU General Public License, please see below.
 
 

 An avr-gcc program for the Atmel ATTiny2313  
 
 Version 1.0   Last Modified:  10/20/2010.  
 
 
 -------------------------------------------------
 USAGE: How to compile and install
 
 
 
 A makefile is provided to compile and install this program using AVR-GCC and avrdude.
 
 To use it, follow these steps:
 1. Update the header of the makefile as needed to reflect the type of AVR programmer that you use.
 2. Open a terminal window and move into the directory with this file and the makefile.  
 3. At the terminal enter
 make clean   <return>
 make all     <return>
 make install <return>
 4. Make sure that avrdude does not report any errors.  If all goes well, the last few lines output by avrdude
 should look something like this:
 
 avrdude: verifying ...
 avrdude: XXXX bytes of flash verified
 
 avrdude: safemode: lfuse reads as 62
 avrdude: safemode: hfuse reads as DF
 avrdude: safemode: efuse reads as FF
 avrdude: safemode: Fuses OK
 
 avrdude done.  Thank you.
 
 
 If you a different programming environment, make sure that you copy over the fuse settings from the makefile.
 
 
 -------------------------------------------------
 
 This code should be relatively straightforward, so not much documentation is provided.  If you'd like to ask 
 questions, suggest improvements, or report success, please use the evilmadscientist forum:
 http://www.evilmadscientist.com/forum/
 
 
 -------------------------------------------------
 
  
*/

#include <avr/io.h> 

#define shortdelay(); 			asm("nop\n\t" \
"nop\n\t");

 

int main (void)
{ 
	uint8_t phase, plateau;
	unsigned int j;
	unsigned int step, stepmax, temp, temp2;
	
//Initialization routine: Clear watchdog timer-- this can prevent several things from going wrong.		
MCUSR &= 0xF7;		//Clear WDRF Flag
WDTCSR	= 0x18;		//Set stupid bits so we can clear timer...
WDTCSR	= 0x00; 
	
	DDRA = 0;
	DDRB = _BV(3);	// output: OC1A
	DDRD = 0;
	
	PORTA = 0;
	PORTB = 0;
	PORTD = 0; 
	
	
asm("cli");		// DISABLE global interrupts
	
	TCCR1B =  0;		// Stop Counter 1
	OCR1A = 8;			// initial value
	ICR1 = 1024; // TOP, initial value
	
	// WGM mode 14: 16-bit fast PWM.  WGM13 = 1, WGM12 = 1, WGM11 = 1, WGM10 = 0.

	TCCR1A = _BV(7) |  _BV(1);		// Clear on compare, set at top
	TCCR1B = _BV(4) |  _BV(3) |  _BV(0);		// Start Counter 1, at speed Clock/1
   	
	stepmax = 256;
	plateau = 20;
	step = 0;
	phase = 1;
	
for (;;)  // main loop
{
	
	
	step++; 
	
	if (step < stepmax) {
		
		if (phase) 
			temp = step; 
		else 
			temp = 256 - step; 
		
	}
	else if (step < (stepmax + plateau))
	{ 
		if (phase) 
			temp = stepmax; 
		else 
			temp = 0; 
	}
	
	else {
		step = 0;
		
		if (phase)
			phase = 0;
		else 
			phase = 1;
		
	}
	
	if (temp > 255) {
		temp = 255;
	}
	temp = (temp * temp) >> 6;
	OCR1A = temp + 8;	
	
	

	
	/*
	 
	 // Linear ramp version for testing only; use stepmax = 1024
	 step++;
	 
	 if (step < stepmax) {
	 
	 if (phase) {
	 OCR1A = step;
	 }
	 else {
	 OCR1A = 1023 - step;
	 }
	 
	 }
	 else {
	 step = 0;
	 
	 if (phase) {
	 phase = 0;
	 }
	 else {
	 phase = 1;
	 }
	 
	 }
	 
	
	 */
	
	
	// delay for timing:
	
	if (phase) {
		j = 0;
		while (j < 475) 
		{
			shortdelay(); 
			j++;		
		}		
	}
	else {
		j = 0;
		while (j < 550) // Slightly slower "exhale"
		{
			shortdelay(); 
			j++;		
		}
	}

	
	

	
	
	
	
	}	//End main loop
	return 0;
}
